@@ -1,13 +1,13 @@
 '''
 Analysing translational movement from image time series.
 
-See the class Movemeter in movemeter_class.py.
-Also see/run the tk_meter.py for GUI.
+See the class Movemeter in movemeter.py.
+Also see/run the tkgui.py for GUI.
 '''
 
 from .version import __version__
 
-from .movemeter_class import Movemeter
+from .movemeter import Movemeter
 from .roi import gen_grid
 
 
